# Copyright 1999-2015. Parallels IP Holdings GmbH. All Rights Reserved.
import xml.etree.ElementTree as ElementTree
from utils import domain_idna_to_ascii, email_idna_to_ascii

class OperabilityCheck:
    def __init__(self, input, output):
        self._input = input
        self._output = output

    def run(self):
        violations = self._validate()
        if violations:
            raise ValidationError(violations)
        issues = self._run()
        for issue in issues:
            result = self._getResult(issue)
            if result:
                self._outputResult(result)

    def _validate(self):
        return []

    def _run(self):
        return []

    def _getResult(self, issue):
        return None

    def _outputResult(self, result):
        self.addIssue(result.severity, result.problem, result.solution, self._output)

    def _validateDomainName(self, domainName, path):
        violations = self._validateRequiredArgument(domainName, path)
        if violations:
            return violations
        try:
            domain_idna_to_ascii(domainName)
        except Exception:
            return [ConstraintViolation(path, u'value \'%s\' is not valid domain name', (domainName))]
        return []

    def _validateEmail(self, email, path):
        violations = self._validateRequiredArgument(email, path)
        if violations:
            return violations
        try:
            email_idna_to_ascii(email)
        except Exception:
            return [ConstraintViolation(path, u'value \'%s\' is not valid email address', (email))]
        return []

    def _validateIpAddress(self, ipAddress, path):
        return self._validateRequiredArgument(ipAddress, path)

    def _validateEnum(self, value, path, allowedValues):
        violations = self._validateRequiredArgument(value, path)
        if violations:
            return violations
        if value not in allowedValues:
            return [ConstraintViolation(path, u'value \'%s\' is not valid, allowed values are: %s', (value, u','.join(allowedValues)))]
        return []

    def _validateRequiredArgument(self, value, path):
        if None == value:
            return [ConstraintViolation(path, u'argument is not found')]
        if 0 == len(value.strip()):
            return [ConstraintViolation(path, u'argument should not be empty')]
        return []

    @staticmethod
    def addIssue(severity, problem, solution, output):
        issueNode = ElementTree.SubElement(output, 'issue')
        ElementTree.SubElement(issueNode, 'severity').text = severity
        ElementTree.SubElement(issueNode, 'description').text = u'%s\n%s' % (problem, solution.rstrip())

class OperabilityCheckResult:
    def __init__(self, severity, problem, solution):
        self.severity = severity
        self.problem = problem
        self.solution = solution

class ConstraintViolation:
    def __init__(self, propertyPath, message, params=()):
        self.propertyPath = propertyPath
        self.message = message
        self.params = params

class ValidationError(Exception):
    def __init__(self, violations):
        self.violations = violations

    def __str__(self):
        invalidProperties = {}
        for violation in self.violations:
            if violation.propertyPath not in invalidProperties:
                invalidProperties[violation.propertyPath] = []
            invalidProperties[violation.propertyPath].append(u' - %s' % (violation.message % violation.params))
        str = u'The following command arguments are invalid:'
        for propertyPath, messages in invalidProperties.iteritems():
            str += u'\n%s:\n%s' % (propertyPath, u'\n'.join(messages))
        return str

class RunError(Exception):
    def __init__(self, message):
        self.message = message

    def __str__(self):
        return self.message

