# Copyright 1999-2015. Parallels IP Holdings GmbH. All Rights Reserved.
import logging
from parallels.operability_check import DomainServicesOperabilityChecker
from parallels.operability_check import DomainFTPService, User, FTPAuthIssueType, Severity
from utils import domain_idna_to_ascii
from OperabilityCheck import OperabilityCheck, OperabilityCheckResult, RunError

_logger = logging.getLogger('operability_check.ftp_auth')

class OperabilityCheckFtpAuth(OperabilityCheck):
    def _validate(self):
        violations = []
        violations += self._validateDomainName(self._input.findtext('domain-name'), 'domain-name')
        for i, serverNode in enumerate(self._input.findall('ftp-servers/ftp-server')):
            violations += self._validateFtpServer(serverNode, '/'.join(['ftp-servers/ftp-server', str(i)]))
        for i, recordNode in enumerate(self._input.findall('users/user')):
            violations += self._validateUser(recordNode, '/'.join(['users/user', str(i)]))
        return violations

    def _run(self):
        domainName = self._input.findtext('domain-name')
        _logger.info(u'Check FTP-service of domain %s', domainName);

        try:
            services = []
            domainAsciiName = domain_idna_to_ascii(domainName).encode('UTF-8')
            users = []
            for userNode in self._input.findall('users/user'):
                user = User(userNode.findtext('login'), userNode.findtext('password'))
                _logger.info(u'Check FTP-user \'%s\'', user.login)
                users.append(user)
            for serverNode in self._input.findall('ftp-servers/ftp-server'):
                if 'ftps' == serverNode.findtext('protocol'):
                    _logger.info(u'FTPS protocol checking is not implemented yet, skip it');
                    continue
                service = DomainFTPService(domainAsciiName, serverNode.findtext('ip-address'), users)
                _logger.info(u'Check FTP-users at \'ftp://%s\'', service.web_server_ip)
                services.append(service)
    
            checker = DomainServicesOperabilityChecker()
            issues = checker.check_ftp_auth(services)
            _logger.info(u'Found %d issues', len(issues));
            return issues
        except Exception as e:
            _logger.debug(u"Run is failed:", exc_info=e)
            raise RunError(u'Unable to check the FTP-service of domain \'%s\': %s' % (domainName, unicode(e)))

    def _getResult(self, issue):
        result = self._results.get(issue.category, self._results[FTPAuthIssueType.INTERNAL_ERROR])
        return OperabilityCheckResult(result['severity'], issue.problem, result['solution'])

    def _validateFtpServer(self, serverNode, path):
        violations = []
        violations += self._validateIpAddress(serverNode.findtext('ip-address'), '/'.join([path, 'ip-address']))
        violations += self._validateEnum(serverNode.findtext('protocol'), '/'.join([path, 'protocol']), ['ftp', 'ftps'])
        return violations

    def _validateUser(self, userNode, path):
        violations = []
        violations += self._validateRequiredArgument(userNode.findtext('login'), '/'.join([path, 'login']))
        violations += self._validateRequiredArgument(userNode.findtext('password'), '/'.join([path, 'password']))
        return violations

    _results = {
        FTPAuthIssueType.WEB_IP_NOT_DEFINED: {
            'severity': Severity.WARNING
            , 'solution': u"""\
Make sure that:
1. The subscription is present in the target Plesk.
2. Virtual hosting is enabled for the subscription in the target Plesk."""
        }
        , FTPAuthIssueType.CHECK_FAILED: {
            'severity': Severity.WARNING
            , 'solution': u"""\
Make sure that:
1. The proper username and password are specified for the subscription in the target Plesk.
2. The FTP service is operating properly on the target Plesk."""
        }
        , FTPAuthIssueType.INTERNAL_ERROR: {
            'severity': Severity.INFO
            , 'solution': u"""\
Manually check whether it is possible to log in to the subscription on the target Plesk by FTP."""
        }
    }

