# Copyright 1999-2015. Parallels IP Holdings GmbH. All Rights Reserved.
import logging
from parallels.operability_check import DomainServicesOperabilityChecker
from parallels.operability_check import DomainMailService, User, MailAuthIssueType, Severity
from utils import domain_idna_to_ascii, email_idna_to_ascii
from OperabilityCheck import OperabilityCheck, OperabilityCheckResult, RunError

_logger = logging.getLogger('operability_check.mail_auth')

class OperabilityCheckMailAuth(OperabilityCheck):
    def _validate(self):
        violations = []
        violations += self._validateDomainName(self._input.findtext('domain-name'), 'domain-name')
        for i, serverNode in enumerate(self._input.findall('mail-servers/mail-server')):
            violations += self._validateMailServer(serverNode, '/'.join(['mail-servers/mail-server', str(i)]))
        for i, recordNode in enumerate(self._input.findall('users/user')):
            violations += self._validateUser(recordNode, '/'.join(['users/user', str(i)]))
        return violations

    def _run(self):
        domainName = self._input.findtext('domain-name')
        _logger.info(u'Check mail-service of domain %s', domainName);

        try:
            services = []
            domainAsciiName = domain_idna_to_ascii(domainName).encode('UTF-8')
            users = []
            for userNode in self._input.findall('users/user'):
                login = userNode.findtext('login')
                _logger.info(u'Check mail-user \'%s\'', login)
                users.append(User(
                    email_idna_to_ascii(login).encode('UTF-8')
                    , userNode.findtext('password').encode('UTF-8')
                ))
            for serverNode in self._input.findall('mail-servers/mail-server'):
                service = DomainMailService(domainAsciiName, serverNode.findtext('ip-address'), users)
                _logger.info(u'Check mail-users at \'%s\'', service.mail_server_ip)
                services.append(service)
    
            checker = DomainServicesOperabilityChecker()
            issues = checker.check_mail_auth(services)
            _logger.info(u'Found %d issues', len(issues));
            return issues
        except Exception as e:
            _logger.debug(u"Run is failed:", exc_info=e)
            raise RunError(u'Unable to check the mail-service of domain \'%s\': %s' % (domainName, unicode(e)))

    def _getResult(self, issue):
        result = self._results.get(issue.category, self._results[''])
        return OperabilityCheckResult(result['severity'], issue.problem, result['solution'])

    def _validateMailServer(self, serverNode, path):
        return self._validateIpAddress(serverNode.findtext('ip-address'), '/'.join([path, 'ip-address']))

    def _validateUser(self, userNode, path):
        violations = []
        violations += self._validateEmail(userNode.findtext('login'), '/'.join([path, 'login']))
        violations += self._validateRequiredArgument(userNode.findtext('password'), '/'.join([path, 'password']))
        return violations

    _results = {
        MailAuthIssueType.MAIL_IP_NOT_DEFINED: {
            'severity': Severity.WARNING
            , 'solution': u"""\
Make sure that:
1. The subscription is present in the target Plesk.
2. The mail service is enabled for the subscription in the target Plesk."""
        }
        , MailAuthIssueType.CHECK_FAILED_IMAP: {
            'severity': Severity.WARNING
            , 'solution': u"""\
Make sure that:
1. Mailboxes are present in the target Plesk.
2. The mail service is enabled for the subscription in the target Plesk.
3. The subscription in the target Plesk is not suspended.
4. The target Plesk mail server is working and it accepts connections by IMAP."""
        }
        , MailAuthIssueType.INTERNAL_ERROR_IMAP: {
            'severity': Severity.INFO
            , 'solution': u"""\
Manually check whether it is possible to log in to the mailbox over IMAP."""
        }
        , MailAuthIssueType.CHECK_FAILED_SMTP: {
            'severity': Severity.WARNING
            , 'solution': u"""\
Make sure that:
1. The mailboxes are present in the target Plesk.
2. The mail service is enabled for the subscriptions in the target Plesk.
3. The subscription in the target Plesk is not suspended.
4. The target Plesk mail server is working and it accepts connections by SMTP."""
        }
        , MailAuthIssueType.INTERNAL_ERROR_SMTP: {
            'severity': Severity.INFO
            , 'solution': u"""\
Manually check whether it is possible to authenticate as the mail user over SMTP."""
            }
        , '': {
            'severity': Severity.INFO
            , 'solution': u"""\
Manually check whether it is possible to authenticate as the mail user."""
        }
    }

